@@ -23,6 +23,7 @@ import (
 	"net/http"
 	"os"
 	"runtime"
+	"strconv"
 	"strings"
 	"time"
 
@@ -31,6 +32,7 @@ import (
 	"google.golang.org/api/compute/v1"
 	"google.golang.org/api/googleapi"
 	"github.com/google/subcommands"
+	"github.com/GoogleCloudPlatform/sapagent/internal/configuration"
 	"github.com/GoogleCloudPlatform/sapagent/internal/databaseconnector"
 	"github.com/GoogleCloudPlatform/sapagent/internal/hanabackup"
 	"github.com/GoogleCloudPlatform/sapagent/internal/instanceinfo"
@@ -39,6 +41,7 @@ import (
 	"github.com/GoogleCloudPlatform/sapagent/internal/usagemetrics"
 	"github.com/GoogleCloudPlatform/sapagent/internal/utils/instantsnapshotgroup"
 	"github.com/GoogleCloudPlatform/sapagent/internal/utils/protostruct"
+	"github.com/GoogleCloudPlatform/sapagent/internal/utils/snapshotgroup"
 	"github.com/GoogleCloudPlatform/workloadagentplatform/sharedlibraries/cloudmonitoring"
 	"github.com/GoogleCloudPlatform/workloadagentplatform/sharedlibraries/commandlineexecutor"
 	"github.com/GoogleCloudPlatform/workloadagentplatform/sharedlibraries/gce"
@@ -97,6 +100,14 @@ type (
 		WaitForISGUploadCompletionWithRetry(ctx context.Context, baseURL string) error
 	}
 
+	// SGInterface is the testable equivalent for SGService for SG operations.
+	SGInterface interface {
+		NewService() error
+		CreateSG(ctx context.Context, project string, data []byte) error
+		WaitForSGUploadCompletionWithRetry(ctx context.Context, project, sgName string) error
+		WaitForSGCreationWithRetry(ctx context.Context, project, sgName string) error
+	}
+
 	snapshotOp struct {
 		op   *compute.Operation
 		name string
@@ -104,13 +115,51 @@ type (
 )
 
 const (
-	metricPrefix = "workload.googleapis.com/sap/agent/"
+	metricPrefix   = "workload.googleapis.com/sap/agent/"
+	m2AgentVersion = "3.10"
 )
 
 var (
 	dbFreezeStartTime, workflowStartTime time.Time
 )
 
+// compareVersions returns true if version1 is less than version2.
+// It expects versions in "major.minor" format.
+func compareVersions(version1, version2 string) (bool, error) {
+	v1Parts := strings.Split(version1, ".")
+	v2Parts := strings.Split(version2, ".")
+
+	if len(v1Parts) != 2 || len(v2Parts) != 2 {
+		return false, fmt.Errorf("invalid version format. expected 'major.minor', got: %q and %q", version1, version2)
+	}
+
+	v1Major, err := strconv.Atoi(v1Parts[0])
+	if err != nil {
+		return false, fmt.Errorf("failed to parse major version for %q: %w", version1, err)
+	}
+	v1Minor, err := strconv.Atoi(v1Parts[1])
+	if err != nil {
+		return false, fmt.Errorf("failed to parse minor version for %q: %w", version1, err)
+	}
+
+	v2Major, err := strconv.Atoi(v2Parts[0])
+	if err != nil {
+		return false, fmt.Errorf("failed to parse major version for %q: %w", version2, err)
+	}
+	v2Minor, err := strconv.Atoi(v2Parts[1])
+	if err != nil {
+		return false, fmt.Errorf("failed to parse minor version for %q: %w", version2, err)
+	}
+
+	if v1Major < v2Major {
+		return true, nil
+	}
+	if v1Major > v2Major {
+		return false, nil
+	}
+	return v1Minor < v2Minor, nil
+}
+
 // ISG is a placeholder struct defining fields potentially required
 // for lifecycle management of Instant Snapshot Groups.
 // It is currently placed here, but ideally it would reside within the compute package.
@@ -149,6 +198,7 @@ type Snapshot struct {
 	gceService                             gceInterface
 	computeService                         *compute.Service
 	isgService                             ISGInterface
+	sgService                              SGInterface
 	status                                 bool
 	timeSeriesCreator                      cloudmonitoring.TimeSeriesCreator
 	help                                   bool
@@ -164,8 +214,10 @@ type Snapshot struct {
 	instanceProperties                     *ipb.InstanceProperties
 	cgName                                 string
 	groupSnapshot                          bool
+	sidadmUser                             bool
 	provisionedIops, provisionedThroughput int64
 	oteLogger                              *onetime.OTELogger
+	UseSnapshotGroupWorkflow               bool
 }
 
 // Name implements the subcommand interface for hanadiskbackup.
@@ -224,14 +276,15 @@ func (s *Snapshot) SetFlags(fs *flag.FlagSet) {
 	fs.StringVar(&s.StorageLocation, "storage-location", "", "Cloud Storage multi-region or the region where you want to store your snapshot. (optional) Default: nearby regional or multi-regional location automatically chosen.")
 	fs.StringVar(&s.Description, "snapshot-description", "", "Description of the new snapshot(optional)")
 	fs.BoolVar(&s.SendToMonitoring, "send-metrics-to-monitoring", true, "Send backup related metrics to cloud monitoring. (optional) Default: true")
-	fs.StringVar(&s.LogPath, "log-path", "", "The log path to write the log file (optional), default value is /var/log/google-cloud-sap-agent/hanadiskbackup.log")
+	fs.StringVar(&s.LogPath, "log-path", "", "The log path to write the log file (optional, required if user is sidadm), default value is /var/log/google-cloud-sap-agent/hanadiskbackup.log")
 	fs.BoolVar(&s.help, "h", false, "Displays help")
 	fs.StringVar(&s.LogLevel, "loglevel", "info", "Sets the logging level")
 	fs.StringVar(&s.Labels, "labels", "", "Labels to be added to the disk snapshot")
 	fs.StringVar(&s.groupSnapshotName, "group-snapshot-name", "", "Group Snapshot name override.(optional - defaults to '<consistency-group-name>-yyyymmdd-hhmmss'.)")
 }
 
-// Execute implements the subcommand interface for hanadiskbackup.
+// Execute executes the subcommand interface for hanadiskbackup.
+// TODO: Add handling for Archive Snapshot.
 func (s *Snapshot) Execute(ctx context.Context, f *flag.FlagSet, args ...any) subcommands.ExitStatus {
 	// Help will return before the args are parsed.
 	lp, cp, exitStatus, completed := onetime.Init(ctx, onetime.InitOptions{
@@ -270,6 +323,14 @@ func (s *Snapshot) Run(ctx context.Context, opts *onetime.RunOptions) (string, s
 	}
 	s.timeSeriesCreator = mc
 
+	s.UseSnapshotGroupWorkflow = true
+	isOlder, err := compareVersions(configuration.AgentVersion, m2AgentVersion)
+	if err != nil {
+		log.CtxLogger(ctx).Warnw("Could not compare agent versions, defaulting to use snapshot group workflow.", "agent_version", configuration.AgentVersion, "m2_agent_version", m2AgentVersion, "error", err)
+	} else if isOlder {
+		s.UseSnapshotGroupWorkflow = false
+	}
+
 	message, exitStatus := s.snapshotHandler(ctx, gce.NewGCEClient, onetime.NewComputeService, hanabackup.CheckDataDir, opts.CloudProperties)
 	if exitStatus != subcommands.ExitSuccess {
 		return message, subcommands.ExitFailure
@@ -290,11 +351,15 @@ func (s *Snapshot) snapshotHandler(ctx context.Context, gceServiceCreator onetim
 		return errMessage, subcommands.ExitFailure
 	}
 
+	// If the command is run as a non-root user, physical data path will be empty but error is nil.
 	if s.hanaDataPath, s.logicalDataPath, s.physicalDataPath, err = checkDataDir(ctx, commandlineexecutor.ExecuteCommand); err != nil {
 		errMessage := "ERROR: Failed to check preconditions"
 		s.oteLogger.LogErrorToFileAndConsole(ctx, errMessage, err)
 		return errMessage, subcommands.ExitFailure
 	}
+	if s.physicalDataPath == "" {
+		s.sidadmUser = true
+	}
 
 	if msg, exitStatus := s.validateDisks(ctx, cp, commandlineexecutor.ExecuteCommand); exitStatus != subcommands.ExitSuccess {
 		return msg, exitStatus
@@ -391,7 +456,7 @@ func (s *Snapshot) snapshotHandler(ctx context.Context, gceServiceCreator onetim
 func (s *Snapshot) validateDisks(ctx context.Context, cp *ipb.CloudProperties, exec commandlineexecutor.Execute) (string, subcommands.ExitStatus) {
 	var isScaleout bool
 	var err error
-	if isScaleout, err = hanabackup.CheckTopology(ctx, exec, s.Sid); err != nil {
+	if isScaleout, err = hanabackup.CheckTopology(ctx, exec, s.Sid, s.sidadmUser); err != nil {
 		errMessage := "ERROR: Failed to check if topology is scaleout or scaleup"
 		s.oteLogger.LogErrorToFileAndConsole(ctx, errMessage, err)
 		return errMessage, subcommands.ExitFailure
@@ -420,13 +485,20 @@ func (s *Snapshot) validateScaleoutDisks(ctx context.Context, cp *ipb.CloudPrope
 		return errMessage, subcommands.ExitFailure
 	}
 
+	if s.UseSnapshotGroupWorkflow {
+		s.sgService = &snapshotgroup.SGService{}
+		if err := s.sgService.NewService(); err != nil {
+			errMessage := "ERROR: Failed to create Snapshot Group service"
+			s.oteLogger.LogErrorToFileAndConsole(ctx, errMessage, err)
+			return errMessage, subcommands.ExitFailure
+		}
+	}
 	s.isgService = &instantsnapshotgroup.ISGService{}
 	if err := s.isgService.NewService(); err != nil {
 		errMessage := "ERROR: Failed to create Instant Snapshot Group service"
 		s.oteLogger.LogErrorToFileAndConsole(ctx, errMessage, err)
 		return errMessage, subcommands.ExitFailure
 	}
-	s.groupSnapshot = true
 
 	disks := strings.Split(s.Disks, ",")
 	for i := range disks {
@@ -438,6 +510,7 @@ func (s *Snapshot) validateScaleoutDisks(ctx context.Context, cp *ipb.CloudPrope
 		return errMessage, subcommands.ExitFailure
 	}
 
+	s.groupSnapshot = true
 	return "", subcommands.ExitSuccess
 }
 
@@ -474,6 +547,12 @@ func (s *Snapshot) validateScaleupDisks(ctx context.Context, cp *ipb.CloudProper
 				s.disks = append(s.disks, disk)
 			}
 		} else {
+			if s.sidadmUser {
+				errMessage := "ERROR: Cannot autodiscover data backing disks"
+				s.oteLogger.LogErrorToFileAndConsole(ctx, errMessage, fmt.Errorf("running as sidadm user"))
+				return errMessage, subcommands.ExitFailure
+			}
+
 			log.CtxLogger(ctx).Info("Reading disk mapping for /hana/data/")
 			if err := s.readDiskMapping(ctx, cp); err != nil {
 				errMessage := "ERROR: Failed to read disk mapping"
@@ -483,9 +562,32 @@ func (s *Snapshot) validateScaleupDisks(ctx context.Context, cp *ipb.CloudProper
 		}
 
 		if len(s.disks) > 1 {
-			if msg, exitStatus := s.verifyStriping(ctx, exec); exitStatus != subcommands.ExitSuccess {
-				return msg, exitStatus
+			if !s.sidadmUser {
+				if msg, exitStatus := s.verifyStriping(ctx, exec); exitStatus != subcommands.ExitSuccess {
+					return msg, exitStatus
+				}
 			}
+
+			if s.UseSnapshotGroupWorkflow {
+				s.sgService = &snapshotgroup.SGService{}
+				if err := s.sgService.NewService(); err != nil {
+					errMessage := "ERROR: Failed to create Snapshot Group service"
+					s.oteLogger.LogErrorToFileAndConsole(ctx, errMessage, err)
+					return errMessage, subcommands.ExitFailure
+				}
+			}
+			s.isgService = &instantsnapshotgroup.ISGService{}
+			if err := s.isgService.NewService(); err != nil {
+				errMessage := "ERROR: Failed to create Instant Snapshot Group service"
+				s.oteLogger.LogErrorToFileAndConsole(ctx, errMessage, err)
+				return errMessage, subcommands.ExitFailure
+			}
+			if err := s.validateDisksBelongToCG(ctx, s.disks); err != nil {
+				errMessage := "ERROR: Failed to validate whether disks belong to consistency group"
+				s.oteLogger.LogErrorToFileAndConsole(ctx, errMessage, err)
+				return errMessage, subcommands.ExitFailure
+			}
+			s.groupSnapshot = true
 		}
 		log.CtxLogger(ctx).Infow("Successfully read disk mapping for /hana/data/", "disks", s.disks, "cgPath", s.cgName, "groupSnapshot", s.groupSnapshot)
 	}
@@ -506,18 +608,6 @@ func (s *Snapshot) verifyStriping(ctx context.Context, exec commandlineexecutor.
 		return errMessage, subcommands.ExitFailure
 	}
 
-	s.isgService = &instantsnapshotgroup.ISGService{}
-	if err := s.isgService.NewService(); err != nil {
-		errMessage := "ERROR: Failed to create Instant Snapshot Group service"
-		s.oteLogger.LogErrorToFileAndConsole(ctx, errMessage, err)
-		return errMessage, subcommands.ExitFailure
-	}
-	if err := s.validateDisksBelongToCG(ctx, s.disks); err != nil {
-		errMessage := "ERROR: Failed to validate whether disks belong to consistency group"
-		s.oteLogger.LogErrorToFileAndConsole(ctx, errMessage, err)
-		return errMessage, subcommands.ExitFailure
-	}
-	s.groupSnapshot = true
 	return "", subcommands.ExitSuccess
 }
 
