@@ -30,11 +30,13 @@ import (
 	"google.golang.org/api/compute/v1"
 	"google.golang.org/api/option"
 	"github.com/google/subcommands"
+	"github.com/GoogleCloudPlatform/sapagent/internal/configuration"
 	"github.com/GoogleCloudPlatform/sapagent/internal/hanabackup"
 	"github.com/GoogleCloudPlatform/sapagent/internal/instanceinfo"
 	"github.com/GoogleCloudPlatform/sapagent/internal/onetime"
 	"github.com/GoogleCloudPlatform/sapagent/internal/usagemetrics"
 	"github.com/GoogleCloudPlatform/sapagent/internal/utils/protostruct"
+	"github.com/GoogleCloudPlatform/sapagent/internal/utils/snapshotgroup"
 	"github.com/GoogleCloudPlatform/workloadagentplatform/sharedlibraries/cloudmonitoring"
 	"github.com/GoogleCloudPlatform/workloadagentplatform/sharedlibraries/commandlineexecutor"
 	"github.com/GoogleCloudPlatform/workloadagentplatform/sharedlibraries/gce"
@@ -76,16 +78,63 @@ type (
 		RemoveResourcePolicies(ctx context.Context, project, zone, diskName string, resourcePolicies []string) (*compute.Operation, error)
 		SetLabels(ctx context.Context, project, zone, diskName, labelFingerprint string, labels map[string]string) (*compute.Operation, error)
 	}
+
+	// SGInterface is the testable equivalent for snapshotgroup.SGService.
+	SGInterface interface {
+		NewService() error
+		BulkInsertFromSG(ctx context.Context, project, zone string, data []byte) (*compute.Operation, error)
+		GetSG(ctx context.Context, project, sgName string) (*snapshotgroup.SGItem, error)
+		ListSnapshotsFromSG(ctx context.Context, project, sgName string) ([]snapshotgroup.SnapshotItem, error)
+		ListDisksFromSnapshot(ctx context.Context, project, zone, snapshot string) ([]snapshotgroup.DiskItem, error)
+	}
 )
 
 const (
-	metricPrefix = "workload.googleapis.com/sap/agent/"
+	metricPrefix   = "workload.googleapis.com/sap/agent/"
+	m2AgentVersion = "3.10"
 )
 
 var (
 	workflowStartTime time.Time
 )
 
+// compareVersions returns true if version1 is less than version2.
+// It expects versions in "major.minor" format.
+func compareVersions(version1, version2 string) (bool, error) {
+	v1Parts := strings.Split(version1, ".")
+	v2Parts := strings.Split(version2, ".")
+
+	if len(v1Parts) != 2 || len(v2Parts) != 2 {
+		return false, fmt.Errorf("invalid version format. expected 'major.minor', got: %q and %q", version1, version2)
+	}
+
+	v1Major, err := strconv.Atoi(v1Parts[0])
+	if err != nil {
+		return false, fmt.Errorf("failed to parse major version for %q: %w", version1, err)
+	}
+	v1Minor, err := strconv.Atoi(v1Parts[1])
+	if err != nil {
+		return false, fmt.Errorf("failed to parse minor version for %q: %w", version1, err)
+	}
+
+	v2Major, err := strconv.Atoi(v2Parts[0])
+	if err != nil {
+		return false, fmt.Errorf("failed to parse major version for %q: %w", version2, err)
+	}
+	v2Minor, err := strconv.Atoi(v2Parts[1])
+	if err != nil {
+		return false, fmt.Errorf("failed to parse minor version for %q: %w", version2, err)
+	}
+
+	if v1Major < v2Major {
+		return true, nil
+	}
+	if v1Major > v2Major {
+		return false, nil
+	}
+	return v1Minor < v2Minor, nil
+}
+
 type (
 	multiDisks struct {
 		disk         *ipb.Disk
@@ -101,6 +150,7 @@ type (
 		DataDiskVG                                                 string
 		gceService                                                 gceInterface
 		computeService                                             *compute.Service
+		sgService                                                  SGInterface
 		cgName                                                     string
 		baseDataPath, baseLogPath                                  string
 		logicalDataPath, logicalLogPath                            string
@@ -121,6 +171,8 @@ type (
 		ProvisionedIops, ProvisionedThroughput, DiskSizeGb         int64
 		IIOTEParams                                                *onetime.InternallyInvokedOTE
 		oteLogger                                                  *onetime.OTELogger
+		UseSnapshotGroupWorkflow                                   bool
+		snapshotItems                                              []snapshotgroup.SnapshotItem
 	}
 )
 
@@ -176,7 +228,8 @@ func (r *Restorer) SetFlags(fs *flag.FlagSet) {
 	fs.StringVar(&r.LogLevel, "loglevel", "info", "Sets the logging level")
 }
 
-// Execute implements the subcommand interface for hanadiskrestore.
+// Execute executes the subcommand interface for hanadiskrestore.
+// TODO: Add handling for Archive Snapshot.
 func (r *Restorer) Execute(ctx context.Context, f *flag.FlagSet, args ...any) subcommands.ExitStatus {
 	// Help will return before the args are parsed.
 	_, cp, exitStatus, completed := onetime.Init(ctx, onetime.InitOptions{
@@ -196,6 +249,14 @@ func (r *Restorer) Execute(ctx context.Context, f *flag.FlagSet, args ...any) su
 
 // Run performs the functionality specified by the hanadiskrestore subcommand.
 func (r *Restorer) Run(ctx context.Context, runOpts *onetime.RunOptions) subcommands.ExitStatus {
+	r.UseSnapshotGroupWorkflow = true
+	isOlder, err := compareVersions(configuration.AgentVersion, m2AgentVersion)
+	if err != nil {
+		log.CtxLogger(ctx).Warnw("Could not compare agent versions, defaulting to use snapshot group workflow.", "agent_version", configuration.AgentVersion, "m2_agent_version", m2AgentVersion, "error", err)
+	} else if isOlder {
+		r.UseSnapshotGroupWorkflow = false
+	}
+
 	r.oteLogger = onetime.CreateOTELogger(runOpts.DaemonMode)
 	return r.restoreHandler(ctx, monitoring.NewMetricClient, gce.NewGCEClient, onetime.NewComputeService, runOpts.CloudProperties, hanabackup.CheckDataDir, hanabackup.CheckLogDir)
 }
@@ -266,6 +327,15 @@ func (r *Restorer) restoreHandler(ctx context.Context, mcc metricClientCreator,
 		return subcommands.ExitFailure
 	}
 
+	if r.UseSnapshotGroupWorkflow {
+		r.sgService = &snapshotgroup.SGService{}
+		if err := r.sgService.NewService(); err != nil {
+			errMessage := "ERROR: Failed to create Snapshot Group service"
+			r.oteLogger.LogErrorToFileAndConsole(ctx, errMessage, err)
+			return subcommands.ExitFailure
+		}
+	}
+
 	log.CtxLogger(ctx).Infow("Starting HANA disk snapshot restore", "sid", r.Sid)
 	r.oteLogger.LogUsageAction(usagemetrics.HANADiskRestore)
 
@@ -583,31 +653,8 @@ func (r *Restorer) checkPreConditions(ctx context.Context, cp *ipb.CloudProperti
 	}
 
 	// Verify the snapshot is present.
-	if !r.isGroupSnapshot {
-		if r.computeService == nil {
-			return fmt.Errorf("compute service is nil")
-		}
-		snapshot, err := r.computeService.Snapshots.Get(r.Project, r.SourceSnapshot).Do()
-		if err != nil {
-			return fmt.Errorf("failed to check if source-snapshot=%v is present: %v", r.SourceSnapshot, err)
-		}
-		r.extractLabels(ctx, snapshot)
-	} else {
-		snapshotList, err := r.gceService.ListSnapshots(ctx, r.Project)
-		if err != nil {
-			return fmt.Errorf("failed to list snapshots: %v", err)
-		}
-
-		var numOfSnapshots int
-		for _, snapshot := range snapshotList.Items {
-			if snapshot.Labels["goog-sapagent-isg"] == r.GroupSnapshot {
-				r.extractLabels(ctx, snapshot)
-				numOfSnapshots++
-			}
-		}
-		if numOfSnapshots != len(r.disks) {
-			return fmt.Errorf("did not get required number of snapshots for restoration, wanted: %v, got: %v", len(r.disks), numOfSnapshots)
-		}
+	if err := r.verifySnapshotPresence(ctx); err != nil {
+		return err
 	}
 
 	if r.isGroupSnapshot && r.NewDiskPrefix != "" {
@@ -647,6 +694,50 @@ func (r *Restorer) checkPreConditions(ctx context.Context, cp *ipb.CloudProperti
 	return nil
 }
 
+// verifySnapshotPresence checks if the source snapshot(s) exist and extracts labels.
+func (r *Restorer) verifySnapshotPresence(ctx context.Context) error {
+	if !r.isGroupSnapshot {
+		if r.computeService == nil {
+			return fmt.Errorf("compute service is nil")
+		}
+		snapshot, err := r.computeService.Snapshots.Get(r.Project, r.SourceSnapshot).Do()
+		if err != nil {
+			return fmt.Errorf("failed to check if source-snapshot=%v is present: %v", r.SourceSnapshot, err)
+		}
+		r.extractLabels(ctx, snapshot)
+		return nil
+	}
+
+	// Group snapshot workflow
+	if r.UseSnapshotGroupWorkflow {
+		sItems, err := r.sgService.ListSnapshotsFromSG(ctx, r.Project, r.GroupSnapshot)
+		if err != nil {
+			return fmt.Errorf("failed to list snapshots from snapshot group: %v", err)
+		}
+		if len(sItems) != len(r.disks) {
+			return fmt.Errorf("did not get required number of snapshots for restoration, wanted: %v, got: %v", len(r.disks), len(sItems))
+		}
+		r.snapshotItems = sItems
+	} else {
+		snapshotList, err := r.gceService.ListSnapshots(ctx, r.Project)
+		if err != nil {
+			return fmt.Errorf("failed to list snapshots: %v", err)
+		}
+
+		var numOfSnapshots int
+		for _, snapshot := range snapshotList.Items {
+			if snapshot.Labels["goog-sapagent-isg"] == r.GroupSnapshot {
+				r.extractLabels(ctx, snapshot)
+				numOfSnapshots++
+			}
+		}
+		if numOfSnapshots != len(r.disks) {
+			return fmt.Errorf("did not get required number of snapshots for restoration, wanted: %v, got: %v", len(r.disks), numOfSnapshots)
+		}
+	}
+	return nil
+}
+
 func (r *Restorer) extractLabels(ctx context.Context, snapshot *compute.Snapshot) {
 	for key, value := range snapshot.Labels {
 		switch key {
