@@ -32,6 +32,7 @@ import (
 	"google.golang.org/api/option"
 	"github.com/google/subcommands"
 	"github.com/GoogleCloudPlatform/sapagent/internal/onetime"
+	"github.com/GoogleCloudPlatform/sapagent/internal/utils/snapshotgroup"
 	ipb "github.com/GoogleCloudPlatform/sapagent/protos/instanceinfo"
 	"github.com/GoogleCloudPlatform/workloadagentplatform/sharedlibraries/cloudmonitoring"
 	cmFake "github.com/GoogleCloudPlatform/workloadagentplatform/sharedlibraries/cloudmonitoring/fake"
@@ -41,6 +42,38 @@ import (
 	"github.com/GoogleCloudPlatform/workloadagentplatform/sharedlibraries/log"
 )
 
+type fakeSGService struct {
+	ListSnapshotsFromSGErr  error
+	ListSnapshotsFromSGResp []snapshotgroup.SnapshotItem
+	GetSGErr                error
+	GetSGResp               *snapshotgroup.SGItem
+	BulkInsertFromSGErr     error
+	ListDisksFromSnapshotFn func(ctx context.Context, project, zone, snapshot string) ([]snapshotgroup.DiskItem, error)
+}
+
+func (f *fakeSGService) NewService() error {
+	return nil
+}
+
+func (f *fakeSGService) BulkInsertFromSG(ctx context.Context, project, zone string, data []byte) (*compute.Operation, error) {
+	return nil, f.BulkInsertFromSGErr
+}
+
+func (f *fakeSGService) GetSG(ctx context.Context, project, sgName string) (*snapshotgroup.SGItem, error) {
+	return f.GetSGResp, f.GetSGErr
+}
+
+func (f *fakeSGService) ListSnapshotsFromSG(ctx context.Context, project, sgName string) ([]snapshotgroup.SnapshotItem, error) {
+	return f.ListSnapshotsFromSGResp, f.ListSnapshotsFromSGErr
+}
+
+func (f *fakeSGService) ListDisksFromSnapshot(ctx context.Context, project, zone, snapshot string) ([]snapshotgroup.DiskItem, error) {
+	if f.ListDisksFromSnapshotFn != nil {
+		return f.ListDisksFromSnapshotFn(ctx, project, zone, snapshot)
+	}
+	return nil, nil
+}
+
 type fakeDiskMapper struct {
 	deviceName []string
 	callCount  int
@@ -56,6 +89,92 @@ func TestMain(t *testing.M) {
 	os.Exit(t.Run())
 }
 
+func TestCompareVersions(t *testing.T) {
+	tests := []struct {
+		name    string
+		v1      string
+		v2      string
+		want    bool
+		wantErr bool
+	}{
+		{
+			name:    "v1 less than v2 minor",
+			v1:      "3.9",
+			v2:      "3.10",
+			want:    true,
+			wantErr: false,
+		},
+		{
+			name:    "v1 greater than v2 minor",
+			v1:      "3.10",
+			v2:      "3.9",
+			want:    false,
+			wantErr: false,
+		},
+		{
+			name:    "v1 equal to v2",
+			v1:      "3.9",
+			v2:      "3.9",
+			want:    false,
+			wantErr: false,
+		},
+		{
+			name:    "v1 less than v2 major",
+			v1:      "2.10",
+			v2:      "3.1",
+			want:    true,
+			wantErr: false,
+		},
+		{
+			name:    "v1 greater than v2 major",
+			v1:      "4.1",
+			v2:      "3.10",
+			want:    false,
+			wantErr: false,
+		},
+		{
+			name:    "invalid v1 format",
+			v1:      "3.9.1",
+			v2:      "3.10",
+			want:    false,
+			wantErr: true,
+		},
+		{
+			name:    "invalid v2 format",
+			v1:      "3.9",
+			v2:      "3",
+			want:    false,
+			wantErr: true,
+		},
+		{
+			name:    "non-integer in v1",
+			v1:      "a.9",
+			v2:      "3.10",
+			want:    false,
+			wantErr: true,
+		},
+		{
+			name:    "non-integer in v2",
+			v1:      "3.9",
+			v2:      "3.b",
+			want:    false,
+			wantErr: true,
+		},
+	}
+
+	for _, tc := range tests {
+		t.Run(tc.name, func(t *testing.T) {
+			got, err := compareVersions(tc.v1, tc.v2)
+			if (err != nil) != tc.wantErr {
+				t.Errorf("compareVersions(%q, %q) returned error: %v, wantErr: %v", tc.v1, tc.v2, err, tc.wantErr)
+			}
+			if got != tc.want {
+				t.Errorf("compareVersions(%q, %q) = %v, want: %v", tc.v1, tc.v2, got, tc.want)
+			}
+		})
+	}
+}
+
 var (
 	defaultRestorer = Restorer{
 		Project:        "my-project",
@@ -1378,6 +1497,64 @@ func TestPrepare(t *testing.T) {
 	}
 }
 
+func TestVerifySnapshotPresence(t *testing.T) {
+	tests := []struct {
+		name    string
+		r       *Restorer
+		wantErr error
+	}{
+		{
+			name: "GroupSnapshotWorkflowWithListErr",
+			r: &Restorer{
+				isGroupSnapshot:          true,
+				UseSnapshotGroupWorkflow: true,
+				sgService: &fakeSGService{
+					ListSnapshotsFromSGErr: cmpopts.AnyError,
+				},
+			},
+			wantErr: cmpopts.AnyError,
+		},
+		{
+			name: "GroupSnapshotWorkflowSnapshotMismatch",
+			r: &Restorer{
+				isGroupSnapshot:          true,
+				UseSnapshotGroupWorkflow: true,
+				disks: []*multiDisks{
+					&multiDisks{},
+				},
+				sgService: &fakeSGService{
+					ListSnapshotsFromSGResp: []snapshotgroup.SnapshotItem{},
+				},
+			},
+			wantErr: cmpopts.AnyError,
+		},
+		{
+			name: "GroupSnapshotWorkflowSuccess",
+			r: &Restorer{
+				isGroupSnapshot:          true,
+				UseSnapshotGroupWorkflow: true,
+				disks: []*multiDisks{
+					&multiDisks{},
+				},
+				sgService: &fakeSGService{
+					ListSnapshotsFromSGResp: []snapshotgroup.SnapshotItem{
+						snapshotgroup.SnapshotItem{},
+					},
+				},
+			},
+			wantErr: nil,
+		},
+	}
+	for _, tc := range tests {
+		t.Run(tc.name, func(t *testing.T) {
+			err := tc.r.verifySnapshotPresence(context.Background())
+			if diff := cmp.Diff(err, tc.wantErr, cmpopts.EquateErrors()); diff != "" {
+				t.Errorf("verifySnapshotPresence() returned diff (-want +got):\n%s", diff)
+			}
+		})
+	}
+}
+
 func TestReadDiskMapping(t *testing.T) {
 	tests := []struct {
 		name     string
