@@ -19,14 +19,17 @@ package hanadiskrestore
 import (
 	"context"
 	"testing"
+	"time"
 
 	"github.com/google/go-cmp/cmp"
 	"github.com/google/go-cmp/cmp/cmpopts"
 	"google.golang.org/api/compute/v1"
 	"github.com/GoogleCloudPlatform/sapagent/internal/onetime"
-	ipb "github.com/GoogleCloudPlatform/sapagent/protos/instanceinfo"
+	"github.com/GoogleCloudPlatform/sapagent/internal/utils/snapshotgroup"
 	"github.com/GoogleCloudPlatform/workloadagentplatform/sharedlibraries/commandlineexecutor"
 	"github.com/GoogleCloudPlatform/workloadagentplatform/sharedlibraries/gce/fake"
+
+	ipb "github.com/GoogleCloudPlatform/sapagent/protos/instanceinfo"
 )
 
 var (
@@ -277,6 +280,9 @@ func TestRestoreFromGroupSnapshot(t *testing.T) {
 							{
 								Name:       "test-snapshot-1",
 								SourceDisk: "test-disk-1",
+								Labels: map[string]string{
+									"goog-sapagent-isg": "test-group-snapshot",
+								},
 							},
 						},
 					},
@@ -295,52 +301,28 @@ func TestRestoreFromGroupSnapshot(t *testing.T) {
 			snapshotKey: "test-snapshot-key",
 			want:        cmpopts.AnyError,
 		},
-		{
-			name: "LVMRenameErrDetachErr",
-			r: &Restorer{
-				GroupSnapshot: "test-group-snapshot",
-				gceService: &fake.TestGCE{
-					SnapshotList: &compute.SnapshotList{
-						Items: []*compute.Snapshot{
-							{
-								Name:       "test-snapshot-1",
-								SourceDisk: "test-disk-1",
-							},
-						},
-					},
-					SnapshotListErr:           nil,
-					DiskAttachedToInstanceErr: nil,
-					IsDiskAttached:            false,
-					DetachDiskErr:             cmpopts.AnyError,
-				},
-			},
-			exec: func(context.Context, commandlineexecutor.Params) commandlineexecutor.Result {
-				return commandlineexecutor.Result{
-					ExitCode: 1,
-					Error:    cmpopts.AnyError,
-				}
-			},
-			cp:          defaultCloudProperties,
-			snapshotKey: "test-snapshot-key",
-			want:        cmpopts.AnyError,
-		},
 		{
 			name: "LVMRenameErr",
 			r: &Restorer{
 				GroupSnapshot: "test-group-snapshot",
+				DataDiskVG:    "test-vg",
 				gceService: &fake.TestGCE{
 					SnapshotList: &compute.SnapshotList{
 						Items: []*compute.Snapshot{
 							{
 								Name:       "test-snapshot-1",
 								SourceDisk: "test-disk-1",
+								Labels: map[string]string{
+									"goog-sapagent-isg": "test-group-snapshot",
+								},
 							},
 						},
 					},
-					SnapshotListErr:           nil,
-					DiskAttachedToInstanceErr: nil,
-					IsDiskAttached:            false,
-					DetachDiskErr:             nil,
+					SnapshotListErr:                  nil,
+					DiskAttachedToInstanceErr:        nil,
+					IsDiskAttached:                   true,
+					DiskAttachedToInstanceDeviceName: "test-device",
+					DetachDiskErr:                    nil,
 				},
 			},
 			exec: func(context.Context, commandlineexecutor.Params) commandlineexecutor.Result {
@@ -1137,3 +1119,341 @@ func TestScaleoutDisksAttachedToInstanceErrors(t *testing.T) {
 		})
 	}
 }
+
+func TestGroupRestoreWithSGWorkflow(t *testing.T) {
+	tests := []struct {
+		name    string
+		r       *Restorer
+		wantErr error
+	}{
+		{
+			name: "GetSGErr",
+			r: &Restorer{
+				sgService: &fakeSGService{
+					GetSGErr: cmpopts.AnyError,
+				},
+			},
+			wantErr: cmpopts.AnyError,
+		},
+		{
+			name: "SGNotFound",
+			r: &Restorer{
+				sgService: &fakeSGService{},
+			},
+			wantErr: cmpopts.AnyError,
+		},
+		{
+			name: "BulkInsertErr",
+			r: &Restorer{
+				sgService: &fakeSGService{
+					GetSGResp:           &snapshotgroup.SGItem{},
+					BulkInsertFromSGErr: cmpopts.AnyError,
+				},
+			},
+			wantErr: cmpopts.AnyError,
+		},
+		{
+			name: "ListDiskErr",
+			r: &Restorer{
+				snapshotItems: []snapshotgroup.SnapshotItem{
+					{Name: "snapshot-1"},
+				},
+				sgService: &fakeSGService{
+					GetSGResp: &snapshotgroup.SGItem{},
+					ListDisksFromSnapshotFn: func(ctx context.Context, project, zone, snapshot string) ([]snapshotgroup.DiskItem, error) {
+						return nil, cmpopts.AnyError
+					},
+				},
+				gceService: &fake.TestGCE{
+					DiskOpErr: nil,
+				},
+			},
+			wantErr: cmpopts.AnyError,
+		},
+		{
+			name: "NoDiskFound",
+			r: &Restorer{
+				snapshotItems: []snapshotgroup.SnapshotItem{
+					{Name: "snapshot-1"},
+				},
+				sgService: &fakeSGService{
+					GetSGResp: &snapshotgroup.SGItem{},
+					ListDisksFromSnapshotFn: func(ctx context.Context, project, zone, snapshot string) ([]snapshotgroup.DiskItem, error) {
+						return nil, nil
+					},
+				},
+				gceService: &fake.TestGCE{
+					DiskOpErr: nil,
+				},
+			},
+			wantErr: cmpopts.AnyError,
+		},
+		{
+			name: "AttachDiskErr",
+			r: &Restorer{
+				snapshotItems: []snapshotgroup.SnapshotItem{
+					{Name: "snapshot-1", SourceDisk: "source-disk-1"},
+				},
+				disks: []*multiDisks{
+					{disk: &ipb.Disk{DiskName: "source-disk-1"}, instanceName: "instance-1"},
+				},
+				sgService: &fakeSGService{
+					GetSGResp: &snapshotgroup.SGItem{},
+					ListDisksFromSnapshotFn: func(ctx context.Context, project, zone, snapshot string) ([]snapshotgroup.DiskItem, error) {
+						return []snapshotgroup.DiskItem{
+							{Name: "new-disk-1", CreationTimestamp: time.Now().Format(time.RFC3339)},
+						}, nil
+					},
+				},
+				gceService: &fake.TestGCE{
+					DiskOpErr:     nil,
+					AttachDiskErr: cmpopts.AnyError,
+				},
+			},
+			wantErr: cmpopts.AnyError,
+		},
+		{
+			name: "CouldNotFindInstance",
+			r: &Restorer{
+				snapshotItems: []snapshotgroup.SnapshotItem{
+					{Name: "snapshot-1", SourceDisk: "source-disk-1"},
+				},
+				disks: []*multiDisks{
+					{disk: &ipb.Disk{DiskName: "random-disk"}, instanceName: "instance-1"},
+				},
+				sgService: &fakeSGService{
+					GetSGResp: &snapshotgroup.SGItem{},
+					ListDisksFromSnapshotFn: func(ctx context.Context, project, zone, snapshot string) ([]snapshotgroup.DiskItem, error) {
+						return []snapshotgroup.DiskItem{
+							{Name: "new-disk-1", CreationTimestamp: time.Now().Format(time.RFC3339)},
+						}, nil
+					},
+				},
+				gceService: &fake.TestGCE{
+					DiskOpErr:     nil,
+					AttachDiskErr: nil,
+				},
+			},
+			wantErr: cmpopts.AnyError,
+		},
+		{
+			name: "DiskAttachedToInstanceErr",
+			r: &Restorer{
+				snapshotItems: []snapshotgroup.SnapshotItem{
+					{Name: "snapshot-1", SourceDisk: "source-disk-1"},
+				},
+				disks: []*multiDisks{
+					{disk: &ipb.Disk{DiskName: "source-disk-1"}, instanceName: "instance-1"},
+				},
+				sgService: &fakeSGService{
+					GetSGResp: &snapshotgroup.SGItem{},
+					ListDisksFromSnapshotFn: func(ctx context.Context, project, zone, snapshot string) ([]snapshotgroup.DiskItem, error) {
+						return []snapshotgroup.DiskItem{
+							{Name: "new-disk-1", CreationTimestamp: time.Now().Format(time.RFC3339)},
+						}, nil
+					},
+				},
+				gceService: &fake.TestGCE{
+					DiskOpErr:                 nil,
+					AttachDiskErr:             nil,
+					DiskAttachedToInstanceErr: cmpopts.AnyError,
+				},
+			},
+			wantErr: cmpopts.AnyError,
+		},
+		{
+			name: "DiskNotAttached",
+			r: &Restorer{
+				snapshotItems: []snapshotgroup.SnapshotItem{
+					{Name: "snapshot-1", SourceDisk: "source-disk-1"},
+				},
+				disks: []*multiDisks{
+					{disk: &ipb.Disk{DiskName: "source-disk-1"}, instanceName: "instance-1"},
+				},
+				sgService: &fakeSGService{
+					GetSGResp: &snapshotgroup.SGItem{},
+					ListDisksFromSnapshotFn: func(ctx context.Context, project, zone, snapshot string) ([]snapshotgroup.DiskItem, error) {
+						return []snapshotgroup.DiskItem{
+							{Name: "new-disk-1", CreationTimestamp: time.Now().Format(time.RFC3339)},
+						}, nil
+					},
+				},
+				gceService: &fake.TestGCE{
+					DiskOpErr:                 nil,
+					AttachDiskErr:             nil,
+					DiskAttachedToInstanceErr: nil,
+					IsDiskAttached:            false,
+				},
+			},
+			wantErr: cmpopts.AnyError,
+		},
+		{
+			name: "Success",
+			r: &Restorer{
+				snapshotItems: []snapshotgroup.SnapshotItem{
+					{Name: "snapshot-1", SourceDisk: "source-disk-1"},
+				},
+				disks: []*multiDisks{
+					{disk: &ipb.Disk{DiskName: "source-disk-1"}, instanceName: "instance-1"},
+				},
+				sgService: &fakeSGService{
+					GetSGResp: &snapshotgroup.SGItem{},
+					ListDisksFromSnapshotFn: func(ctx context.Context, project, zone, snapshot string) ([]snapshotgroup.DiskItem, error) {
+						return []snapshotgroup.DiskItem{
+							{Name: "new-disk-1", CreationTimestamp: time.Now().Format(time.RFC3339)},
+						}, nil
+					},
+				},
+				gceService: &fake.TestGCE{
+					DiskOpErr:                 nil,
+					AttachDiskErr:             nil,
+					IsDiskAttached:            true,
+					DiskAttachedToInstanceErr: nil,
+					AddResourcePoliciesErr:    nil,
+				},
+			},
+			wantErr: nil,
+		},
+	}
+
+	for _, tc := range tests {
+		t.Run(tc.name, func(t *testing.T) {
+			err := tc.r.groupRestoreWithSGWorkflow(context.Background(), commandlineexecutor.ExecuteCommand, defaultCloudProperties)
+			if diff := cmp.Diff(err, tc.wantErr, cmpopts.EquateErrors()); diff != "" {
+				t.Errorf("groupRestoreWithSGWorkflow() returned diff (-want +got):\n%s", diff)
+			}
+		})
+	}
+}
+
+func TestRenameLVMForScaleup(t *testing.T) {
+	tests := []struct {
+		name     string
+		r        *Restorer
+		exec     commandlineexecutor.Execute
+		cp       *ipb.CloudProperties
+		diskName string
+		wantErr  error
+	}{
+		{
+			name: "isScaleout",
+			r: &Restorer{
+				isScaleout: true,
+			},
+			wantErr: nil,
+		},
+		{
+			name: "EmptyDataDiskVG",
+			r: &Restorer{
+				isScaleout: false,
+				gceService: &fake.TestGCE{
+					IsDiskAttached: true,
+				},
+				DataDiskVG: "",
+			},
+			cp:      defaultCloudProperties,
+			wantErr: nil,
+		},
+		{
+			name: "renameLVMErrDetachErr",
+			r: &Restorer{
+				isScaleout: false,
+				gceService: &fake.TestGCE{
+					IsDiskAttached: true,
+					GetInstanceResp: []*compute.Instance{{
+						Disks: []*compute.AttachedDisk{
+							{DeviceName: "dev", Source: "projects/p/zones/z/disks/d"},
+						},
+					}},
+					GetInstanceErr: []error{nil},
+					ListDisksResp: []*compute.DiskList{{
+						Items: []*compute.Disk{
+							{Name: "d"},
+						},
+					}},
+					ListDisksErr:  []error{nil},
+					DetachDiskErr: cmpopts.AnyError,
+				},
+				DataDiskVG: "vg",
+			},
+			exec: func(ctx context.Context, params commandlineexecutor.Params) commandlineexecutor.Result {
+				return commandlineexecutor.Result{Error: cmpopts.AnyError}
+			},
+			cp:      defaultCloudProperties,
+			wantErr: cmpopts.AnyError,
+		},
+		{
+			name: "renameLVMErr",
+			r: &Restorer{
+				isScaleout: false,
+				gceService: &fake.TestGCE{
+					IsDiskAttached: true,
+					GetInstanceResp: []*compute.Instance{{
+						Disks: []*compute.AttachedDisk{
+							{DeviceName: "dev", Source: "projects/p/zones/z/disks/d"},
+						},
+					}},
+					GetInstanceErr: []error{nil},
+					ListDisksResp: []*compute.DiskList{{
+						Items: []*compute.Disk{
+							{Name: "d"},
+						},
+					}},
+					ListDisksErr:  []error{nil},
+					DetachDiskErr: nil,
+				},
+				DataDiskVG: "vg",
+			},
+			exec: func(ctx context.Context, params commandlineexecutor.Params) commandlineexecutor.Result {
+				return commandlineexecutor.Result{Error: cmpopts.AnyError}
+			},
+			cp:      defaultCloudProperties,
+			wantErr: cmpopts.AnyError,
+		},
+		{
+			name: "Success",
+			r: &Restorer{
+				isScaleout: false,
+				gceService: &fake.TestGCE{
+					IsDiskAttached: true,
+					GetInstanceResp: []*compute.Instance{{
+						Disks: []*compute.AttachedDisk{
+							{DeviceName: "dev", Source: "projects/p/zones/z/disks/d"},
+						},
+					}},
+					GetInstanceErr: []error{nil},
+					ListDisksResp: []*compute.DiskList{{
+						Items: []*compute.Disk{
+							{Name: "d"},
+						},
+					}},
+					ListDisksErr: []error{nil},
+				},
+				DataDiskVG: "vg",
+			},
+			exec: func(ctx context.Context, params commandlineexecutor.Params) commandlineexecutor.Result {
+				return commandlineexecutor.Result{
+					StdOut: "PV         VG    Fmt  Attr PSize   PFree\n/dev/sdd  vg lvm2 a--  500.00g 300.00g",
+				}
+			},
+			cp:      defaultCloudProperties,
+			wantErr: nil,
+		},
+	}
+
+	for _, tc := range tests {
+		t.Run(tc.name, func(t *testing.T) {
+			disks := []multiDisks{
+				{
+					disk: &ipb.Disk{
+						DiskName: "d",
+					},
+				},
+			}
+			err := tc.r.renameLVMForScaleup(context.Background(), tc.exec, tc.cp, disks)
+			if diff := cmp.Diff(err, tc.wantErr, cmpopts.EquateErrors()); diff != "" {
+				t.Errorf("renameLVMForScaleup() returned diff (-want +got):\n%s", diff)
+			}
+		})
+	}
+}
