@@ -410,6 +410,7 @@ func (s *Status) agentStatus(ctx context.Context) (*spb.AgentStatus, *cpb.Config
 	if err != nil && runtime.GOOS == "linux" {
 		log.CtxLogger(ctx).Errorw("Could not fetch kernel version", "error", err)
 	}
+	agentStatus.InstanceUri = fmt.Sprintf("projects/%s/zones/%s/instances/%s", s.CloudProps.GetProjectId(), s.CloudProps.GetZone(), s.CloudProps.GetInstanceId())
 
 	return agentStatus, config
 }
@@ -460,9 +461,9 @@ func (s *Status) processMetricsStatus(ctx context.Context, config *cpb.Configura
 		State: spb.State_UNSPECIFIED_STATE,
 		ConfigValues: []*spb.ConfigValue{
 			configValue("collect_process_metrics", config.GetCollectionConfiguration().GetCollectProcessMetrics(), false),
-			configValue("process_metrics_frequency", config.GetCollectionConfiguration().GetProcessMetricsFrequency(), 5),
+			configValue("process_metrics_frequency", config.GetCollectionConfiguration().GetProcessMetricsFrequency(), 30),
 			configValue("process_metrics_to_skip", config.GetCollectionConfiguration().GetProcessMetricsToSkip(), []string{}),
-			configValue("slow_process_metrics_frequency", config.GetCollectionConfiguration().GetSlowProcessMetricsFrequency(), 30),
+			configValue("slow_process_metrics_frequency", config.GetCollectionConfiguration().GetSlowProcessMetricsFrequency(), 120),
 		},
 	}
 	if !config.GetCollectionConfiguration().GetCollectProcessMetrics() {
@@ -581,6 +582,10 @@ func (s *Status) systemDiscoveryStatus(ctx context.Context, config *cpb.Configur
 			return logCheckFailureAndReturnStatus(ctx, status, err.Error(), spb.State_FAILURE_STATE)
 		}
 	}
+	// Ensure usr/sap has executable permissions.
+	if err := checkFilePermissions("/usr/sap", 0100, s.stat); err != nil {
+		return logCheckFailureAndReturnStatus(ctx, status, "/usr/sap needs to be executable. Run 'sudo chmod +x /usr/sap'", spb.State_FAILURE_STATE)
+	}
 	status.FullyFunctional = spb.State_SUCCESS_STATE
 	return status
 }
